=begin
   Copyright 2012, Atalanta Systems LTD

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

=end

require "vmtools/version"
require 'vagrant'
require 'erb'
require 'mixlib/config'
require 'mixlib/cli'

module VMTools
class VMConfig
  extend(Mixlib::Config)
  configure do |c|
    c[:orgname] = 'atalanta'
    c[:knife_config] = '/home/jakshi/Dropbox/atalanta-systems/git-repos/atalanta-chef-repo/.chef/knife.rb'
    c[:vtemplate] = 'Vagrantfile.erb'
    c[:vconfig] = 'Vagranfile'
    c[:homedir] = "#{ENV['HOME']}"
    c[:testbox_name] = 'testbox'
    c[:vm_type] = 'vagrant'
    c[:vmemory] = "768"
    c[:vforward_ports] = {}
    c[:vbox_name] = "precise32"
    c[:vbox_url] = "http://files.vagrantup.com/precise32.box"
  end
end

class VMCLI

  include Mixlib::CLI

  option :create_vm,
    :short => "-c",
    :long => "--create",
    :description => "Create new VM according to config file"

  option :delete_vm,
    :short => "-d",
    :long => "--delete",
    :description => "Delete VM according to config file"

  option :recreate_vm,
    :short => "-r",
    :long => "--recreate",
    :description => "Delete current VM and then create new VM according to config file"

  option :version,
    :short => "-v",
    :long => "--version",
    :description => "Print version info"
  
  option :help,
    :short => "-h",
    :long => "--help",
    :description => "Show this message",
    :on => :tail,
    :boolean => true,
    :show_options => true,
    :exit => 0
end

class VM
  def get_binding # this is only a helper method to access the objects
  # binding method
    binding
  end
  def initialize
    # Get vmtools gem path
    spec = Gem::Specification.find_by_name("vmtools")
    @gem_root = spec.gem_dir
    # Read config.
    # First searching for config in current dir and then in ruby gems dir
    begin
      VMConfig.from_file('vmtconfig.rb')
    rescue Errno::ENOENT
      puts "Warring: There's no vmtconfig.rb in current dir. Using default vmtconfig.rb"
      VMConfig.from_file("#{@gem_root}/config/vmtconfig.rb")
    end

    # Now only vagrant type of VMs is supported.
    if VMConfig.vm_type != 'vagrant'
      puts "Well, this's embarassing, but only vagrant VMs are supported now"
      puts "Your specified #{VMConfig.vm_type} VM in config."
      exit(0)
    end
    # Assign class instance variables
    ## Chef stuff
    @orgname = VMConfig.orgname
    @knife_config = VMConfig.knife_config

    ## Common stuff
    @testbox_name = VMConfig.testbox_name

    ## Vagrant stuff
    @vtemplate = VMConfig.vtemplate
    @vconfig = VMConfig.vconfig
    @homedir = VMConfig.homedir
    @vmemory = VMConfig.vmemory
    @vforward_ports = VMConfig.vforward_ports
    @vbox_name = VMConfig.vbox_name
    @vbox_url = VMConfig.vbox_url
  end

  def run
    # Proccess command line parameters
    ## If no arguments - then let's print help
    if ARGV.empty?
      puts "Well, this's embarassing, but you need to specify an option! Please read help:"
      ARGV << "-h"
    end

    cli = VMCLI.new
    banner = <<-eos
Vmtools is util for create/delete/recreate test VM
VM configuration stored in vmtconfig.rb file

Usage:
       vmtools --create|--delete|--recreate

Options:
eos
    cli.banner=banner
    ## Parse options
    begin
      cli.parse_options
    rescue OptionParser::InvalidOption
      # if we have wrong option, print help.
      puts "Well, this's embarassing, but wrong option! Please read help:"
      ARGV.clear << "-h"
      retry
    end
    cli.config.each do |opt,value|
      if value
        case opt
          when :create_vm
            self.create
          when :delete_vm
            self.delete
          when :recreate_vm
            self.delete
            self.create
          when :version
          puts "vmtools #{VMTools::VERSION} (c) 2012 Atalanta Systems"
        end
      end
    end
  end

  def create
    # Read template content
    # First searching for config in current dir and then in ruby gems dir
    begin
      template_content = IO.read(@vtemplate)
    rescue Errno::ENOENT
      puts "Warring: There's no Vagrantfile.erb in current dir. Using default Vagrantfile.erb"
      template_content = IO.read("#{@gem_root}/templates/Vagrantfile.erb")
    end

    erb = ERB.new(template_content)

    env = Vagrant::Environment.new
    # Clean out old vagrant config before writing a new one
    begin
      File.delete(@vconfig)
    rescue Errno::ENOENT
      puts "Warring: There's no #{@vconfig} in this dir. Nothing to delete"
    end
    # Generate new config from erb template
    IO.write(@vconfig,erb.result(self.get_binding))
    env.cli("up")
  end

  def delete
    # Delete chef client/node
    ENV['ORGNAME'] = "#{@orgname}"
    system("knife node delete #{@testbox} -c #{@knife_config} -y")
    system("knife client delete #{@testbox} -c #{@knife_config} -y")
    # Delete VM  
    env = Vagrant::Environment.new
    begin
      File.delete(@vconfig)
    rescue Errno::ENOENT
      puts "Warring: There's no #{@vconfig} in this dir. Nothing to delete"
    end
    env.cli("destroy","--force")
  end

end

# Module end
end
